@@ -1,6 +1,6 @@
 <?php
 
-class Autologon_Plugin extends PHPUnit_Framework_TestCase
+class Autologon_Plugin extends PHPUnit\Framework\TestCase
 {
 
     function setUp()
@@ -18,6 +18,10 @@ class Autologon_Plugin extends PHPUnit_Framework_TestCase
 
         $this->assertInstanceOf('autologon', $plugin);
         $this->assertInstanceOf('rcube_plugin', $plugin);
+
+        // TODO
+        $plugin->startup([]);
+        $plugin->authenticate([]);
     }
 }
 
