@@ -1,6 +1,6 @@
 <?php
 
-class FilesystemAttachments_Plugin extends PHPUnit_Framework_TestCase
+class FilesystemAttachments_Plugin extends PHPUnit\Framework\TestCase
 {
 
     function setUp()
