@@ -1,11 +1,11 @@
 <?php
 
-class KrbAuthentication_Plugin extends PHPUnit_Framework_TestCase
+class KrbAuthentication_Plugin extends PHPUnit\Framework\TestCase
 {
 
     function setUp()
     {
-        include_once dirname(__FILE__) . '/../krb_authentication.php';
+        include_once __DIR__ . '/../krb_authentication.php';
     }
 
     /**
@@ -20,4 +20,3 @@ class KrbAuthentication_Plugin extends PHPUnit_Framework_TestCase
         $this->assertInstanceOf('rcube_plugin', $plugin);
     }
 }
-
