@@ -25,7 +25,7 @@ class rcube_sieve_forward extends rcube_sieve_engine
 {
     protected $error;
     protected $script_name;
-    protected $forward = array();
+    protected $forward = [];
 
     function actions()
     {
@@ -38,9 +38,9 @@ class rcube_sieve_forward extends rcube_sieve_engine
         }
 
         $this->plugin->add_label('forward.saving');
-        $this->rc->output->add_handlers(array(
-            'forwardform' => array($this, 'forward_form'),
-        ));
+        $this->rc->output->add_handlers([
+                'forwardform' => [$this, 'forward_form'],
+        ]);
 
         $this->rc->output->set_pagetitle($this->plugin->gettext('forward'));
         $this->rc->output->send('managesieve.forward');
@@ -61,7 +61,7 @@ class rcube_sieve_forward extends rcube_sieve_engine
 
         $list     = $this->list_scripts();
         $master   = $this->rc->config->get('managesieve_kolab_master');
-        $included = array();
+        $included = [];
 
         $this->script_name = false;
 
@@ -130,15 +130,17 @@ class rcube_sieve_forward extends rcube_sieve_engine
             return;
         }
 
-        $list   = array();
+        $list   = [];
         $active = in_array($this->script_name, $this->active);
 
         // find (first) simple forward rule that can be expressed with the minimal settings
         foreach ($this->script as $idx => $rule) {
             if (empty($this->forward) && !empty($rule['actions']) && $rule['actions'][0]['type'] == 'redirect') {
                 $ignore_rule = false;
-                $target = null;
-                $stop_found = false;
+                $target      = null;
+                $stop_found  = false;
+                $action      = 'keep';
+
                 foreach ($rule['actions'] as $act) {
                     if ($stop_found) {
                         // we might loose information if there rules after the stop
@@ -148,7 +150,7 @@ class rcube_sieve_forward extends rcube_sieve_engine
                         $action = 'copy';
                     }
                     else if ($act['type'] == 'stop') {
-                        // we might loose information if there rules after the stop
+                        // we might loose information if there are rules after the stop
                         $stop_found = true;
                     }
                     else if ($act['type'] == 'discard') {
@@ -160,7 +162,7 @@ class rcube_sieve_forward extends rcube_sieve_engine
                             $ignore_rule = true;
                         }
                         else {
-                            $action = $act['copy'] ? 'copy' : 'redirect';
+                            $action = !empty($act['copy']) ? 'copy' : 'redirect';
                             $target = $act['target'];
                         }
                     }
@@ -176,14 +178,14 @@ class rcube_sieve_forward extends rcube_sieve_engine
                 }
 
                 if (!$ignore_rule) {
-                    $this->forward = array_merge($rule['actions'][0], array(
+                    $this->forward = array_merge($rule['actions'][0], [
                         'idx'      => $idx,
                         'disabled' => $rule['disabled'] || !$active,
                         'name'     => $rule['name'],
                         'tests'    => $rule['tests'],
-                        'action'   => $action ?: 'keep',
+                        'action'   => $action,
                         'target'   => $target,
-                    ));
+                    ]);
                 }
             }
             else if ($active) {
@@ -206,46 +208,46 @@ class rcube_sieve_forward extends rcube_sieve_engine
         $target_domain = rcube_utils::get_input_value('action_domain', rcube_utils::INPUT_POST);
 
         $date_extension = in_array('date', $this->exts);
-        $forward_tests  = (array) $this->forward['tests'];
 
-        if ($action == 'redirect' || $action == 'copy') {
-            if ($target_domain) {
-                $target .= '@' . $target_domain;
-            }
-            if (empty($target) || !rcube_utils::check_email($target)) {
-                $error = 'noemailwarning';
-            }
+        if ($target_domain) {
+            $target .= '@' . $target_domain;
         }
 
-        if (empty($forward_tests)) {
-            $forward_tests = (array) $this->rc->config->get('managesieve_forward_test', array(array('test' => 'true')));
+        if (empty($target) || !rcube_utils::check_email($target)) {
+            $error = 'noemailwarning';
         }
 
-        if (!$error) {
+        if (empty($this->forward['tests'])) {
+            $forward_tests = (array) $this->rc->config->get('managesieve_forward_test', [['test' => 'true']]);
+        }
+        else {
+            $forward_tests = (array) $this->forward['tests'];
+        }
+
+        if (empty($error)) {
             $rule               = $this->forward;
             $rule['type']       = 'if';
-            $rule['name']       = $rule['name'] ?: $this->plugin->gettext('forward');
+            $rule['name']       = !empty($rule['name']) ? $rule['name'] : $this->plugin->gettext('forward');
             $rule['disabled']   = $status == 'off';
             $rule['tests']      = $forward_tests;
             $rule['join']       = $date_extension ? count($forward_tests) > 1 : false;
-            $rule['actions']    = array();
-            $rule['after']      = $after;
-
-            if ($action && $action != 'keep') {
-                $rule['actions'][] = array(
-                    'type'   => $action == 'discard' ? 'discard' : 'redirect',
+            $rule['actions']    = [[
+                    'type'   => 'redirect',
                     'copy'   => $action == 'copy',
-                    'target' => $action != 'discard' ? $target : '',
-                );
-            }
+                    'target' => $target,
+            ]];
 
-            if ($this->save_forward_script($rule)) {
+            if ($this->merge_rule($rule, $this->forward, $this->script_name)) {
                 $this->rc->output->show_message('managesieve.forwardsaved', 'confirmation');
                 $this->rc->output->send();
             }
         }
 
-        $this->rc->output->show_message($error ?: 'managesieve.saveerror', 'error');
+        if (empty($error)) {
+            $error = 'managesieve.saveerror';
+        }
+
+        $this->rc->output->show_message($error, 'error');
         $this->rc->output->send();
     }
 
@@ -255,20 +257,21 @@ class rcube_sieve_forward extends rcube_sieve_engine
     public function forward_form($attrib)
     {
         // build FORM tag
-        $form_id = $attrib['id'] ?: 'form';
-        $out     = $this->rc->output->request_form(array(
-            'id'      => $form_id,
-            'name'    => $form_id,
-            'method'  => 'post',
-            'task'    => 'settings',
-            'action'  => 'plugin.managesieve-forward',
-            'noclose' => true
-            ) + $attrib);
+        $form_id = !empty($attrib['id']) ? $attrib['id'] : 'form';
+        $out     = $this->rc->output->request_form([
+                'id'      => $form_id,
+                'name'    => $form_id,
+                'method'  => 'post',
+                'task'    => 'settings',
+                'action'  => 'plugin.managesieve-forward',
+                'noclose' => true
+            ] + $attrib
+        );
 
 
         // form elements
-        $status = new html_select(array('name' => 'forward_status', 'id' => 'forward_status'));
-        $action = new html_select(array('name' => 'forward_action', 'id' => 'forward_action'));
+        $status = new html_select(['name' => 'forward_status', 'id' => 'forward_status', 'class' => 'custom-select']);
+        $action = new html_select(['name' => 'forward_action', 'id' => 'forward_action', 'class' => 'custom-select']);
 
         $status->add($this->plugin->gettext('forward.on'), 'on');
         $status->add($this->plugin->gettext('forward.off'), 'off');
@@ -280,15 +283,16 @@ class rcube_sieve_forward extends rcube_sieve_engine
 
         // force domain selection in redirect email input
         $domains  = (array) $this->rc->config->get('managesieve_domains');
-        $redirect = $this->forward['action'] == 'redirect' || $this->forward['action'] == 'copy';
+        $redirect = !empty($this->forward['action'])
+            && ($this->forward['action'] == 'redirect' || $this->forward['action'] == 'copy');
 
         if (!empty($domains)) {
             sort($domains);
 
-            $domain_select = new html_select(array('name' => 'action_domain', 'id' => 'action_domain'));
+            $domain_select = new html_select(['name' => 'action_domain', 'id' => 'action_domain', 'class' => 'custom-select']);
             $domain_select->add(array_combine($domains, $domains));
 
-            if ($redirect && $this->forward['target']) {
+            if ($redirect && !empty($this->forward['target'])) {
                 $parts = explode('@', $this->forward['target']);
                 if (!empty($parts)) {
                     $this->forward['domain'] = array_pop($parts);
@@ -301,16 +305,18 @@ class rcube_sieve_forward extends rcube_sieve_engine
         $action_target = '<span id="action_target_span" class="input-group">'
             . '<input type="text" name="action_target" id="action_target"'
             . ' value="' .($redirect ? rcube::Q($this->forward['target'], 'strict', false) : '') . '"'
-            . (!empty($domains) ? ' size="20"' : ' size="35"') . '/>'
-            . (!empty($domains) ? ' <span class="input-group-prepend input-group-append"><span class="input-group-text">@</span></span> '
-                . $domain_select->show($this->forward['domain']) : '')
+            . (!empty($domain_select) ? ' size="20"' : ' size="35"') . '/>'
+            . (!empty($domain_select) ? ' <span class="input-group-prepend input-group-append"><span class="input-group-text">@</span></span> '
+                . $domain_select->show(!empty($this->forward['domain']) ? $this->forward['domain'] : null) : '')
             . '</span>';
 
         // Message tab
-        $table = new html_table(array('cols' => 2));
+        $table = new html_table(['cols' => 2]);
 
         $table->add('title', html::label('forward_action', $this->plugin->gettext('forward.action')));
-        $table->add('forward input-group input-group-combo', $action->show($this->forward['action']) . ' ' . $action_target);
+        $table->add('forward input-group input-group-combo',
+            $action->show(!empty($this->forward['action']) ? $this->forward['action'] : null) . ' ' . $action_target
+        );
 
         $table->add('title', html::label('forward_status', $this->plugin->gettext('forward.status')));
         $table->add(null, $status->show(!isset($this->forward['disabled']) || $this->forward['disabled'] ? 'off' : 'on'));
@@ -324,99 +330,6 @@ class rcube_sieve_forward extends rcube_sieve_engine
         return $out;
     }
 
-    /**
-     * Saves forward script (adding some variables)
-     */
-    protected function save_forward_script($rule)
-    {
-        // if script does not exist create a new one
-        if ($this->script_name === null || $this->script_name === false) {
-            $this->script_name = $this->rc->config->get('managesieve_script_name');
-            if (empty($this->script_name)) {
-                $this->script_name = 'roundcube';
-            }
-
-            // use default script contents
-            if (!$this->rc->config->get('managesieve_kolab_master')) {
-                $script_file = $this->rc->config->get('managesieve_default');
-                if ($script_file && is_readable($script_file)) {
-                    $content = file_get_contents($script_file);
-                }
-            }
-
-            // create and load script
-            if ($this->sieve->save_script($this->script_name, $content)) {
-                $this->sieve->load($this->script_name);
-            }
-        }
-
-        $script_active = in_array($this->script_name, $this->active);
-
-        // re-order rules if needed
-        if (isset($rule['after']) && $rule['after'] !== '') {
-            // reset original forward rule
-            if (isset($this->forward['idx'])) {
-                $this->script[$this->forward['idx']] = null;
-            }
-
-            // add at target position
-            if ($rule['after'] >= count($this->script) - 1) {
-                $this->script[] = $rule;
-            }
-            else {
-                $script = array();
-
-                foreach ($this->script as $idx => $r) {
-                    if ($r) {
-                        $script[] = $r;
-                    }
-
-                    if ($idx == $rule['after']) {
-                        $script[] = $rule;
-                    }
-                }
-
-                $this->script = $script;
-            }
-
-            $this->script = array_values(array_filter($this->script));
-        }
-        // update original forward rule if it exists
-        else if (isset($this->forward['idx'])) {
-            $this->script[$this->forward['idx']] = $rule;
-        }
-        // otherwise put forward rule on top
-        else {
-            array_unshift($this->script, $rule);
-        }
-
-        // if the script was not active, we need to de-activate
-        // all rules except the forward rule, but only if it is not disabled
-        if (!$script_active && !$rule['disabled']) {
-            foreach ($this->script as $idx => $r) {
-                if (empty($r['actions']) || $r['actions'][0]['type'] != 'forward') {
-                    $this->script[$idx]['disabled'] = true;
-                }
-            }
-        }
-
-        if (!$this->sieve->script) {
-            return false;
-        }
-
-        $this->sieve->script->content = $this->script;
-
-        // save the script
-        $saved = $this->save_script($this->script_name);
-
-        // activate the script
-        if ($saved && !$script_active && !$rule['disabled']) {
-            $this->activate_script($this->script_name);
-        }
-
-        return $saved;
-    }
-
     /**
      * API: get forward rule
      *
@@ -428,12 +341,12 @@ class rcube_sieve_forward extends rcube_sieve_engine
         $this->init_script();
         $this->forward_rule();
 
-        $forward = array(
+        $forward = [
             'supported' => $this->exts,
             'enabled'   => empty($this->forward['disabled']),
             'action'    => $this->forward['action'],
             'target'    => $this->forward['target'],
-        );
+        ];
 
         return $forward;
     }
@@ -457,7 +370,7 @@ class rcube_sieve_forward extends rcube_sieve_engine
 
         if ($data['action'] == 'redirect' || $data['action'] == 'copy') {
             if (empty($data['target']) || !rcube_utils::check_email($data['target'])) {
-                $this->error = "Invalid address in action taget: " . $data['target'];
+                $this->error = "Invalid address in action target: " . $data['target'];
                 return false;
             }
         }
@@ -467,26 +380,22 @@ class rcube_sieve_forward extends rcube_sieve_engine
         }
 
         if (empty($forward_tests)) {
-            $forward_tests = (array) $this->rc->config->get('managesieve_forward_test', array(array('test' => 'true')));
+            $forward_tests = (array) $this->rc->config->get('managesieve_forward_test', [['test' => 'true']]);
         }
 
         $rule             = $this->forward;
         $rule['type']     = 'if';
-        $rule['name']     = $rule['name'] ?: 'Out-of-Office';
+        $rule['name']     = !empty($rule['name']) ? $rule['name'] : 'Out-of-Office';
         $rule['disabled'] = isset($data['enabled']) && !$data['enabled'];
         $rule['tests']    = $forward_tests;
         $rule['join']     = $date_extension ? count($forward_tests) > 1 : false;
-        $rule['actions']  = array();
-
-        if ($data['action'] && $data['action'] != 'keep') {
-            $rule['actions'][] = array(
-                'type'   => $data['action'] == 'discard' ? 'discard' : 'redirect',
+        $rule['actions']  = [[
+                'type'   => 'redirect',
                 'copy'   => $data['action'] == 'copy',
-                'target' => $data['action'] != 'discard' ? $data['target'] : '',
-            );
-        }
+                'target' => $data['target'],
+        ]];
 
-        return $this->save_forward_script($rule);
+        return $this->merge_rule($rule, $this->forward, $this->script_name);
     }
 
     /**
