@@ -1,6 +1,6 @@
 <?php
 
-class ShowAdditionalHeaders_Plugin extends PHPUnit_Framework_TestCase
+class ShowAdditionalHeaders_Plugin extends PHPUnit\Framework\TestCase
 {
 
     function setUp()
@@ -20,4 +20,3 @@ class ShowAdditionalHeaders_Plugin extends PHPUnit_Framework_TestCase
         $this->assertInstanceOf('rcube_plugin', $plugin);
     }
 }
-
