@@ -32,36 +32,41 @@ class rcube_message
      *
      * @var rcube
      */
-    private $app;
+    protected $app;
 
     /**
      * Instance of storage class
      *
      * @var rcube_storage
      */
-    private $storage;
+    protected $storage;
 
     /**
      * Instance of mime class
      *
      * @var rcube_mime
      */
-    private $mime;
-    private $opt = array();
-    private $parse_alternative = false;
-    private $tnef_decode = false;
+    protected $mime;
+
+    protected $opt               = [];
+    protected $parse_alternative = false;
+    protected $got_html_part     = false;
+    protected $tnef_decode       = false;
 
     public $uid;
     public $folder;
     public $headers;
     public $sender;
     public $context;
-    public $parts        = array();
-    public $mime_parts   = array();
-    public $inline_parts = array();
-    public $attachments  = array();
+    public $body;
+    public $parts        = [];
+    public $mime_parts   = [];
+    public $inline_parts = [];
+    public $attachments  = [];
     public $subject      = '';
     public $is_safe      = false;
+    public $pgp_mime     = false;
+    public $encrypted_part;
 
     const BODY_MAX_SIZE = 1048576; // 1MB
 
@@ -84,6 +89,7 @@ class rcube_message
             list($uid, $folder) = explode('-', $uid, 2);
         }
 
+        $context = null;
         if (preg_match('/^([0-9]+)\.([0-9.]+)$/', $uid, $matches)) {
             $uid     = $matches[1];
             $context = $matches[2];
@@ -97,7 +103,7 @@ class rcube_message
 
         // Set current folder
         $this->storage->set_folder($this->folder);
-        $this->storage->set_options(array('all_headers' => true));
+        $this->storage->set_options(['all_headers' => true]);
 
         $this->headers = $this->storage->get_message($uid);
 
@@ -107,16 +113,18 @@ class rcube_message
 
         $this->tnef_decode = (bool) $this->app->config->get('tnef_decode', true);
 
-        $this->set_safe($is_safe || $_SESSION['safe_messages'][$this->folder.':'.$uid]);
-        $this->opt = array(
+        $this->set_safe($is_safe || !empty($_SESSION['safe_messages'][$this->folder.':'.$uid]));
+        $this->opt = [
             'safe'        => $this->is_safe,
             'prefer_html' => $this->app->config->get('prefer_html'),
-            'get_url'     => $this->app->url(array(
+            'get_url'     => $this->app->url([
                     'action' => 'get',
                     'mbox'   => $this->folder,
-                    'uid'    => $uid),
-                false, false, true)
-        );
+                    'uid'    => $uid
+                ],
+                false, false, true
+            )
+        ];
 
         if (!empty($this->headers->structure)) {
             $this->get_mime_numbers($this->headers->structure);
@@ -132,7 +140,7 @@ class rcube_message
         $this->sender  = current($from);
 
         // notify plugins and let them analyze this structured message object
-        $this->app->plugins->exec_hook('message_load', array('object' => $this));
+        $this->app->plugins->exec_hook('message_load', ['object' => $this]);
     }
 
     /**
@@ -168,14 +176,16 @@ class rcube_message
      * @param string $mime_id Part MIME-ID
      * @param mixed  $embed Mimetype class for parts to be embedded
      *
-     * @return string URL or false if part does not exist
+     * @return string|false URL or false if part does not exist
      */
     public function get_part_url($mime_id, $embed = false)
     {
-        if ($this->mime_parts[$mime_id])
-            return $this->opt['get_url'] . '&_part=' . $mime_id . ($embed ? '&_embed=1&_mimeclass=' . $embed : '');
-        else
-            return false;
+        if (!empty($this->mime_parts[$mime_id])) {
+            return $this->opt['get_url'] . '&_part=' . $mime_id
+                . ($embed ? '&_embed=1&_mimeclass=' . $embed : '');
+        }
+
+        return false;
     }
 
     /**
@@ -183,9 +193,9 @@ class rcube_message
      *
      * @param string   $mime_id           Part MIME-ID
      * @param resource $fp                File pointer to save the message part
-     * @param boolean  $skip_charset_conv Disables charset conversion
+     * @param bool     $skip_charset_conv Disables charset conversion
      * @param int      $max_bytes         Only read this number of bytes
-     * @param boolean  $formatted         Enables formatting of text/* parts bodies
+     * @param bool     $formatted         Enables formatting of text/* parts bodies
      *
      * @return string Part content
      * @deprecated
@@ -205,7 +215,7 @@ class rcube_message
             $this->storage->set_folder($this->folder);
 
             return $this->storage->get_message_part($this->uid, $mime_id, $part,
-                NULL, $fp, $skip_charset_conv, $max_bytes, $formatted);
+                null, $fp, $skip_charset_conv, $max_bytes, $formatted);
         }
     }
 
@@ -222,13 +232,15 @@ class rcube_message
      */
     public function get_part_body($mime_id, $formatted = false, $max_bytes = 0, $mode = null)
     {
-        if (!($part = $this->mime_parts[$mime_id])) {
+        if (empty($this->mime_parts[$mime_id])) {
             return;
         }
 
+        $part = $this->mime_parts[$mime_id];
+
         // allow plugins to modify part body
         $plugin = $this->app->plugins->exec_hook('message_part_body',
-            array('object' => $this, 'part' => $part));
+            ['object' => $this, 'part' => $part]);
 
         // only text parts can be formatted
         $formatted = $formatted && $part->ctype_primary == 'text';
@@ -241,12 +253,14 @@ class rcube_message
                 null, null, true, 0, false);
         }
 
+        $charset = !empty($this->headers) ? $this->headers->charset : null;
+
         // body stored in message structure (winmail/inline-uuencode)
         if ($part->body !== null || $part->encoding == 'stream') {
             $body = $part->body;
 
             if ($formatted && $body) {
-                $body = self::format_part_body($body, $part, $this->headers->charset);
+                $body = self::format_part_body($body, $part, $charset);
             }
 
             if ($max_bytes && strlen($body) > $max_bytes) {
@@ -286,7 +300,7 @@ class rcube_message
         }
 
         if (!$mode && $body && $formatted) {
-            $body = self::format_part_body($body, $part, $this->headers->charset);
+            $body = self::format_part_body($body, $part, $charset);
         }
 
         return $body;
@@ -312,7 +326,7 @@ class rcube_message
         }
 
         // detect charset...
-        if (!$part->charset || strtoupper($part->charset) == 'US-ASCII') {
+        if (empty($part->charset) || strtoupper($part->charset) == 'US-ASCII') {
             // try to extract charset information from HTML meta tag (#1488125)
             if ($part->ctype_secondary == 'html' && preg_match('/<meta[^>]+charset=([a-z0-9-_]+)/i', $body, $m)) {
                 $part->charset = strtoupper($m[1]);
@@ -371,6 +385,10 @@ class rcube_message
                         return true;
                     }
 
+                    if (empty($this->mime_parts[implode('.', $level)])) {
+                        return true;
+                    }
+
                     $parent = $this->mime_parts[implode('.', $level)];
 
                     if (!$this->check_context($parent)) {
@@ -378,7 +396,7 @@ class rcube_message
                     }
 
                     $max_delta = $depth - (1 + ($last == 'multipart/alternative' ? 1 : 0));
-                    $last      = $parent->real_mimetype ?: $parent->mimetype;
+                    $last      = !empty($parent->real_mimetype) ? $parent->real_mimetype : $parent->mimetype;
 
                     if (!preg_match('/^multipart\/(alternative|related|signed|encrypted|mixed)$/', $last)
                         || ($last == 'multipart/mixed' && $parent_depth < $max_delta)
@@ -415,11 +433,11 @@ class rcube_message
         foreach ($this->mime_parts as $part) {
             if ($part->mimetype == 'text/plain') {
                 // Skip if part is an attachment, don't use is_attachment() here
-                if ($part->filename) {
+                if (!empty($part->filename)) {
                     continue;
                 }
 
-                if (!$part->size) {
+                if (empty($part->size)) {
                     continue;
                 }
 
@@ -461,7 +479,7 @@ class rcube_message
      * @param rcube_message_part &$part    Reference to the part if found
      * @param bool               $enriched Enables checking for text/enriched parts too
      *
-     * @return string HTML message part content
+     * @return string|null HTML message part content
      */
     public function first_html_part(&$part = null, $enriched = false)
     {
@@ -484,7 +502,7 @@ class rcube_message
      * @param rcube_message_part &$part  Reference to the part if found
      * @param bool               $strict Check only text/plain parts
      *
-     * @return string Plain text message/part content
+     * @return string|null Plain text message/part content
      */
     public function first_text_part(&$part = null, $strict = false)
     {
@@ -513,7 +531,7 @@ class rcube_message
             return $this->mime_parts;
         }
 
-        $parts = array();
+        $parts = [];
 
         foreach ($this->mime_parts as $part_id => $part) {
             if ($this->check_context($part)) {
@@ -562,7 +580,8 @@ class rcube_message
                 $this->pgp_mime = true;
             }
             if ($this->pgp_mime && ($mpart->mimetype == 'application/octet-stream' ||
-                    (!empty($mpart->filename) && $mpart->filename != 'version.txt'))) {
+                    (!empty($mpart->filename) && $mpart->filename != 'version.txt'))
+            ) {
                 $this->encrypted_part = $mime_id;
                 return $mpart;
             }
@@ -572,7 +591,7 @@ class rcube_message
     }
 
     /**
-     * Read the message structure returend by the IMAP server
+     * Read the message structure returned by the IMAP server
      * and build flat lists of content parts and attachments
      *
      * @param rcube_message_part $structure Message structure node
@@ -581,17 +600,22 @@ class rcube_message
     private function parse_structure($structure, $recursive = false)
     {
         // real content-type of message/rfc822 part
-        if ($structure->mimetype == 'message/rfc822' && $structure->real_mimetype) {
+        if ($structure->mimetype == 'message/rfc822' && !empty($structure->real_mimetype)) {
             $mimetype = $structure->real_mimetype;
 
             // parse headers from message/rfc822 part
             if (!isset($structure->headers['subject']) && !isset($structure->headers['from'])) {
-                list($headers, ) = explode("\r\n\r\n", $this->get_part_body($structure->mime_id, false, 32768));
+                list($headers, $body) = explode("\r\n\r\n", $this->get_part_body($structure->mime_id, false, 32768), 2);
                 $structure->headers = rcube_mime::parse_headers($headers);
 
                 if ($this->context === $structure->mime_id) {
                     $this->headers = rcube_message_header::from_array($structure->headers);
                 }
+
+                // For small text messages we can optimize, so an additional FETCH is not needed
+                if ($structure->size < 32768 && count($structure->parts) == 1 && $structure->parts[0]->ctype_primary == 'text') {
+                    $structure->parts[0]->body = $body;
+                }
             }
         }
         else {
@@ -599,19 +623,28 @@ class rcube_message
         }
 
         // show message headers
-        if ($recursive && is_array($structure->headers) &&
-            (isset($structure->headers['subject']) || $structure->headers['from'] || $structure->headers['to'])
+        if (
+            $recursive
+            && is_array($structure->headers)
+            && (
+                isset($structure->headers['subject'])
+                || !empty($structure->headers['from'])
+                || !empty($structure->headers['to'])
+            )
         ) {
-            $c = new stdClass;
-            $c->type = 'headers';
+            $c = new rcube_message_part();
+            $c->type    = 'headers';
             $c->headers = $structure->headers;
             $this->add_part($c);
         }
 
         // Allow plugins to handle message parts
-        $plugin = $this->app->plugins->exec_hook('message_part_structure',
-            array('object' => $this, 'structure' => $structure,
-                'mimetype' => $mimetype, 'recursive' => $recursive));
+        $plugin = $this->app->plugins->exec_hook('message_part_structure', [
+                'object'    => $this,
+                'structure' => $structure,
+                'mimetype'  => $mimetype,
+                'recursive' => $recursive
+        ]);
 
         if ($plugin['abort']) {
             return;
@@ -626,7 +659,7 @@ class rcube_message
         // print body if message doesn't have multiple parts
         if ($message_ctype_primary == 'text' && !$recursive) {
             // parts with unsupported type add to attachments list
-            if (!in_array($message_ctype_secondary, array('plain', 'html', 'enriched'))) {
+            if (!in_array($message_ctype_secondary, ['plain', 'html', 'enriched'])) {
                 $this->add_part($structure, 'attachment');
                 return;
             }
@@ -667,16 +700,19 @@ class rcube_message
                 // others as attachments (#1489358)
 
                 // check if sub part is
-                if ($is_multipart)
+                if ($is_multipart) {
                     $related_part = $p;
-                else if ($sub_mimetype == 'text/plain' && !$plain_part)
+                }
+                else if ($sub_mimetype == 'text/plain' && !isset($plain_part)) {
                     $plain_part = $p;
-                else if ($sub_mimetype == 'text/html' && !$html_part) {
+                }
+                else if ($sub_mimetype == 'text/html' && !isset($html_part)) {
                     $html_part = $p;
                     $this->got_html_part = true;
                 }
-                else if ($sub_mimetype == 'text/enriched' && !$enriched_part)
+                else if ($sub_mimetype == 'text/enriched' && !isset($enriched_part)) {
                     $enriched_part = $p;
+                }
                 else {
                     // add unsupported/unrecognized parts to attachments list
                     $this->add_part($sub_part, 'attachment');
@@ -684,25 +720,26 @@ class rcube_message
             }
 
             // parse related part (alternative part could be in here)
-            if ($related_part !== null && !$this->parse_alternative) {
+            if (isset($related_part) && !$this->parse_alternative) {
                 $this->parse_alternative = true;
                 $this->parse_structure($structure->parts[$related_part], true);
                 $this->parse_alternative = false;
 
                 // if plain part was found, we should unset it if html is preferred
-                if ($this->opt['prefer_html'] && count($this->parts)) {
+                if (!empty($this->opt['prefer_html']) && count($this->parts)) {
                     $plain_part = null;
                 }
             }
 
             // choose html/plain part to print
-            if ($html_part !== null && $this->opt['prefer_html']) {
+            $print_part = null;
+            if (isset($html_part) && !empty($this->opt['prefer_html'])) {
                 $print_part = $structure->parts[$html_part];
             }
-            else if ($enriched_part !== null) {
+            else if (isset($enriched_part)) {
                 $print_part = $structure->parts[$enriched_part];
             }
-            else if ($plain_part !== null) {
+            else if (isset($plain_part)) {
                 $print_part = $structure->parts[$plain_part];
             }
 
@@ -711,17 +748,20 @@ class rcube_message
                 $print_part->type = 'content';
 
                 // Allow plugins to handle also this part
-                $plugin = $this->app->plugins->exec_hook('message_part_structure',
-                    array('object' => $this, 'structure' => $print_part,
-                        'mimetype' => $print_part->mimetype, 'recursive' => true));
+                $plugin = $this->app->plugins->exec_hook('message_part_structure', [
+                        'object'    => $this,
+                        'structure' => $print_part,
+                        'mimetype'  => $print_part->mimetype,
+                        'recursive' => true
+                ]);
 
                 if (!$plugin['abort']) {
                     $this->add_part($print_part);
                 }
             }
             // show plaintext warning
-            else if ($html_part !== null && empty($this->parts)) {
-                $c = new stdClass;
+            else if (isset($html_part) && empty($this->parts)) {
+                $c = new rcube_message_part();
                 $c->type            = 'content';
                 $c->ctype_primary   = 'text';
                 $c->ctype_secondary = 'plain';
@@ -731,9 +771,9 @@ class rcube_message
                 $this->add_part($c);
             }
         }
-        // this is an ecrypted message -> create a plaintext body with the according message
+        // this is an encrypted message -> create a plaintext body with the according message
         else if ($mimetype == 'multipart/encrypted') {
-            $p = new stdClass;
+            $p = new rcube_message_part();
             $p->type            = 'content';
             $p->ctype_primary   = 'text';
             $p->ctype_secondary = 'plain';
@@ -744,7 +784,7 @@ class rcube_message
             $this->add_part($p);
 
             // add encrypted payload part as attachment
-            if (is_array($structure->parts)) {
+            if (!empty($structure->parts)) {
                 for ($i=0; $i < count($structure->parts); $i++) {
                     $subpart = $structure->parts[$i];
                     if ($subpart->mimetype == 'application/octet-stream' || !empty($subpart->filename)) {
@@ -753,9 +793,9 @@ class rcube_message
                 }
             }
         }
-        // this is an S/MIME ecrypted message -> create a plaintext body with the according message
+        // this is an S/MIME encrypted message -> create a plaintext body with the according message
         else if ($mimetype == 'application/pkcs7-mime') {
-            $p = new stdClass;
+            $p = new rcube_message_part();
             $p->type            = 'content';
             $p->ctype_primary   = 'text';
             $p->ctype_secondary = 'plain';
@@ -788,13 +828,16 @@ class rcube_message
                     $this->parse_structure($mail_part, true);
                 }
                 // part text/[plain|html] or delivery status
-                else if ((($part_mimetype == 'text/plain' || $part_mimetype == 'text/html') && $mail_part->disposition != 'attachment') ||
-                    in_array($part_mimetype, array('message/delivery-status', 'text/rfc822-headers', 'message/disposition-notification'))
+                else if ((($part_mimetype == 'text/plain' || $part_mimetype == 'text/html') && $mail_part->disposition != 'attachment')
+                    || in_array($part_mimetype, ['message/delivery-status', 'text/rfc822-headers', 'message/disposition-notification'])
                 ) {
                     // Allow plugins to handle also this part
-                    $plugin = $this->app->plugins->exec_hook('message_part_structure',
-                        array('object' => $this, 'structure' => $mail_part,
-                            'mimetype' => $part_mimetype, 'recursive' => true));
+                    $plugin = $this->app->plugins->exec_hook('message_part_structure', [
+                            'object'    => $this,
+                            'structure' => $mail_part,
+                            'mimetype'  => $part_mimetype,
+                            'recursive' => true
+                    ]);
 
                     if ($plugin['abort']) {
                         continue;
@@ -808,9 +851,9 @@ class rcube_message
                     list($primary_type, $secondary_type) = explode('/', $plugin['mimetype']);
 
                     // add text part if it matches the prefs
-                    if (!$this->parse_alternative ||
-                        ($secondary_type == 'html' && $this->opt['prefer_html']) ||
-                        ($secondary_type == 'plain' && !$this->opt['prefer_html'])
+                    if (!$this->parse_alternative
+                        || ($secondary_type == 'html' && $this->opt['prefer_html'])
+                        || ($secondary_type == 'plain' && !$this->opt['prefer_html'])
                     ) {
                         $mail_part->type = 'content';
                         $this->add_part($mail_part);
@@ -864,9 +907,10 @@ class rcube_message
                     }
                 }
                 // part is a file/attachment
-                else if (preg_match('/^(inline|attach)/', $mail_part->disposition) ||
-                    $mail_part->headers['content-id'] ||
-                    ($mail_part->filename &&
+                else if (
+                    preg_match('/^(inline|attach)/', $mail_part->disposition)
+                    || $mail_part->headers['content-id']
+                    || ($mail_part->filename &&
                         (empty($mail_part->disposition) || preg_match('/^[a-z0-9!#$&.+^_-]+$/i', $mail_part->disposition)))
                 ) {
                     // skip apple resource forks
@@ -874,18 +918,19 @@ class rcube_message
                         continue;
                     }
 
-                    if ($mail_part->headers['content-id']) {
-                        $mail_part->content_id = preg_replace(array('/^</', '/>$/'), '', $mail_part->headers['content-id']);
+                    if (!empty($mail_part->headers['content-id'])) {
+                        $mail_part->content_id = preg_replace(['/^</', '/>$/'], '', $mail_part->headers['content-id']);
                     }
 
-                    if ($mail_part->headers['content-location']) {
+                    if (!empty($mail_part->headers['content-location'])) {
                         $mail_part->content_location = $mail_part->headers['content-base'] . $mail_part->headers['content-location'];
                     }
 
                     // part belongs to a related message and is linked
                     // Note: mixed is not supposed to contain inline images, but we've found such examples (#5905)
-                    if (preg_match('/^multipart\/(related|relative|mixed)/', $mimetype)
-                        && ($mail_part->content_id || $mail_part->content_location)
+                    if (
+                        preg_match('/^multipart\/(related|relative|mixed)/', $mimetype)
+                        && (!empty($mail_part->content_id) || !empty($mail_part->content_location))
                     ) {
                         $this->add_part($mail_part, 'inline');
                     }
@@ -923,7 +968,7 @@ class rcube_message
             // if this is a related part try to resolve references
             // Note: mixed is not supposed to contain inline images, but we've found such examples (#5905)
             if (preg_match('/^multipart\/(related|relative|mixed)/', $mimetype) && count($this->inline_parts)) {
-                $a_replaces = array();
+                $a_replaces = [];
                 $img_regexp = '/^image\/(gif|jpe?g|png|tiff|bmp|svg)/';
 
                 foreach ($this->inline_parts as $inline_object) {
@@ -931,7 +976,7 @@ class rcube_message
                     if (isset($inline_object->content_id)) {
                         $a_replaces['cid:'.$inline_object->content_id] = $part_url;
                     }
-                    if ($inline_object->content_location) {
+                    if (!empty($inline_object->content_location)) {
                         $a_replaces[$inline_object->content_location] = $part_url;
                     }
 
@@ -1030,17 +1075,13 @@ class rcube_message
         // @TODO: attachment may be huge, handle body via file
         $body     = $this->get_part_body($part->mime_id);
         $tnef     = new rcube_tnef_decoder;
-        $tnef_arr = $tnef->decompress($body);
-        $parts    = array();
+        $tnef_arr = $tnef->decompress($body, true);
+        $parts    = [];
 
         unset($body);
 
         // HTML body
-        if (
-            !empty($tnef_arr['message'])
-            && !empty($tnef_arr['message']['size'])
-            && $tnef_arr['message']['subtype'] == 'html'
-        ) {
+        if (!empty($tnef_arr['message'])) {
             $tpart = new rcube_message_part;
 
             $tpart->encoding        = 'stream';
@@ -1048,8 +1089,8 @@ class rcube_message
             $tpart->ctype_secondary = 'html';
             $tpart->mimetype        = 'text/html';
             $tpart->mime_id         = 'winmail.' . $part->mime_id . '.html';
-            $tpart->size            = $tnef_arr['message']['size'];
-            $tpart->body            = $tnef_arr['message']['stream'];
+            $tpart->size            = strlen($tnef_arr['message']);
+            $tpart->body            = $tnef_arr['message'];
 
             $parts[] = $tpart;
         }
@@ -1089,7 +1130,7 @@ class rcube_message
     {
         // @TODO: messages may be huge, handle body via file
         $part->body = $this->get_part_body($part->mime_id);
-        $parts      = array();
+        $parts      = [];
         $pid        = 0;
 
         // FIXME: line length is max.65?
