@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -16,7 +17,7 @@
  +-----------------------------------------------------------------------+
 */
 
-// location where plugins are loade from
+// location where plugins are loaded from
 if (!defined('RCUBE_PLUGINS_DIR')) {
     define('RCUBE_PLUGINS_DIR', RCUBE_INSTALL_PATH . 'plugins/');
 }
@@ -31,26 +32,30 @@ class rcube_plugin_api
 {
     static protected $instance;
 
+    /** @var string */
     public $dir;
+    /** @var string */
     public $url = 'plugins/';
+    /** @var string */
     public $task = '';
+    /** @var bool */
     public $initialized = false;
 
     public $output;
-    public $handlers              = array();
-    public $allowed_prefs         = array();
-    public $allowed_session_prefs = array();
-    public $active_plugins        = array();
+    public $handlers              = [];
+    public $allowed_prefs         = [];
+    public $allowed_session_prefs = [];
+    public $active_plugins        = [];
 
-    protected $plugins           = array();
-    protected $plugins_initialized = array();
-    protected $tasks             = array();
-    protected $actions           = array();
-    protected $actionmap         = array();
-    protected $objectsmap        = array();
-    protected $template_contents = array();
-    protected $exec_stack        = array();
-    protected $deprecated_hooks  = array();
+    protected $plugins           = [];
+    protected $plugins_initialized = [];
+    protected $tasks             = [];
+    protected $actions           = [];
+    protected $actionmap         = [];
+    protected $objectsmap        = [];
+    protected $template_contents = [];
+    protected $exec_stack        = [];
+    protected $deprecated_hooks  = [];
 
 
     /**
@@ -81,21 +86,22 @@ class rcube_plugin_api
      * This has to be done after rcmail::load_gui() or rcmail::json_init()
      * was called because plugins need to have access to rcmail->output
      *
-     * @param object rcube Instance of the rcube base class
-     * @param string Current application task (used for conditional plugin loading)
+     * @param rcube  $app  Instance of the rcube base class
+     * @param string $task Current application task (used for conditional plugin loading)
      */
     public function init($app, $task = '')
     {
         $this->task   = $task;
         $this->output = $app->output;
+
         // register an internal hook
-        $this->register_hook('template_container', array($this, 'template_container_hook'));
-        // maybe also register a shudown function which triggers
+        $this->register_hook('template_container', [$this, 'template_container_hook']);
+        // maybe also register a shutdown function which triggers
         // shutdown functions of all plugin objects
 
         foreach ($this->plugins as $plugin) {
             // ... task, request type and framed mode
-            if (!$this->plugins_initialized[$plugin->ID] && !$this->filter($plugin)) {
+            if (empty($this->plugins_initialized[$plugin->ID]) && !$this->filter($plugin)) {
                 $plugin->init();
                 $this->plugins_initialized[$plugin->ID] = $plugin;
             }
@@ -111,17 +117,17 @@ class rcube_plugin_api
      * This has to be done after rcmail::load_gui() or rcmail::json_init()
      * was called because plugins need to have access to rcmail->output
      *
-     * @param array List of configured plugins to load
-     * @param array List of plugins required by the application
+     * @param array $plugins_enabled  List of configured plugins to load
+     * @param array $plugins_required List of plugins required by the application
      */
-    public function load_plugins($plugins_enabled, $required_plugins = array())
+    public function load_plugins($plugins_enabled, $plugins_required = [])
     {
         foreach ($plugins_enabled as $plugin_name) {
             $this->load_plugin($plugin_name);
         }
 
-        // check existance of all required core plugins
-        foreach ($required_plugins as $plugin_name) {
+        // check existence of all required core plugins
+        foreach ($plugins_required as $plugin_name) {
             $loaded = false;
             foreach ($this->plugins as $plugin) {
                 if ($plugin instanceof $plugin_name) {
@@ -137,10 +143,12 @@ class rcube_plugin_api
 
             // trigger fatal error if still not loaded
             if (!$loaded) {
-                rcube::raise_error(array(
-                    'code' => 520, 'type' => 'php',
-                    'file' => __FILE__, 'line' => __LINE__,
-                    'message' => "Requried plugin $plugin_name was not loaded"), true, true);
+                rcube::raise_error([
+                        'code' => 520, 'file' => __FILE__, 'line' => __LINE__,
+                        'message' => "Required plugin $plugin_name was not loaded"
+                    ],
+                    true, true
+                );
             }
         }
     }
@@ -148,11 +156,11 @@ class rcube_plugin_api
     /**
      * Load the specified plugin
      *
-     * @param string  Plugin name
-     * @param boolean Force loading of the plugin even if it doesn't match the filter
-     * @param boolean Require loading of the plugin, error if it doesn't exist
+     * @param string $plugin_name Plugin name
+     * @param bool   $force        Force loading of the plugin even if it doesn't match the filter
+     * @param bool   $require      Require loading of the plugin, error if it doesn't exist
      *
-     * @return boolean True on success, false if not loaded or failure
+     * @return bool True on success, false if not loaded or failure
      */
     public function load_plugin($plugin_name, $force = false, $require = true)
     {
@@ -165,21 +173,28 @@ class rcube_plugin_api
 
         // Validate the plugin name to prevent from path traversal
         if (preg_match('/[^a-zA-Z0-9_-]/', $plugin_name)) {
-            rcube::raise_error(array('code' => 520,
-                    'file' => __FILE__, 'line' => __LINE__,
-                    'message' => "Invalid plugin name: $plugin_name"), true, false);
+            rcube::raise_error([
+                    'code' => 520, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid plugin name: $plugin_name"
+                ],
+                true, false
+            );
+
             return false;
         }
 
         // plugin already loaded?
-        if (!$this->plugins[$plugin_name]) {
+        if (!isset($this->plugins[$plugin_name])) {
             $fn = "$plugins_dir/$plugin_name/$plugin_name.php";
 
             if (!is_readable($fn)) {
                 if ($require) {
-                    rcube::raise_error(array('code' => 520, 'type' => 'php',
-                        'file' => __FILE__, 'line' => __LINE__,
-                        'message' => "Failed to load plugin file $fn"), true, false);
+                    rcube::raise_error([
+                            'code' => 520, 'file' => __FILE__, 'line' => __LINE__,
+                            'message' => "Failed to load plugin file $fn"
+                        ],
+                        true, false
+                    );
                 }
 
                 return false;
@@ -191,10 +206,12 @@ class rcube_plugin_api
 
             // instantiate class if exists
             if (!class_exists($plugin_name, false)) {
-                rcube::raise_error(array('code' => 520, 'type' => 'php',
-                    'file' => __FILE__, 'line' => __LINE__,
-                    'message' => "No plugin class $plugin_name found in $fn"),
-                    true, false);
+                rcube::raise_error([
+                        'code' => 520, 'file' => __FILE__, 'line' => __LINE__,
+                        'message' => "No plugin class $plugin_name found in $fn"
+                    ],
+                    true, false
+                );
 
                 return false;
             }
@@ -218,9 +235,15 @@ class rcube_plugin_api
             }
         }
 
-        if ($plugin = $this->plugins[$plugin_name]) {
+        if (!empty($this->plugins[$plugin_name])) {
+            $plugin = $this->plugins[$plugin_name];
+
             // init a plugin only if $force is set or if we're called after initialization
-            if (($force || $this->initialized) && !$this->plugins_initialized[$plugin_name] && ($force || !$this->filter($plugin))) {
+            if (
+                ($force || $this->initialized)
+                && empty($this->plugins_initialized[$plugin_name])
+                && ($force || !$this->filter($plugin))
+            ) {
                 $plugin->init();
                 $this->plugins_initialized[$plugin_name] = $plugin;
             }
@@ -230,9 +253,10 @@ class rcube_plugin_api
     }
 
     /**
-     * check if we should prevent this plugin from initialising
+     * Check if we should prevent this plugin from initializing
+     *
+     * @param rcube_plugin $plugin Plugin object
      *
-     * @param $plugin
      * @return bool
      */
     private function filter($plugin)
@@ -244,33 +268,27 @@ class rcube_plugin_api
 
     /**
      * Get information about a specific plugin.
-     * This is either provided my a plugin's info() method or extracted from a package.xml or a composer.json file
+     * This is either provided by a plugin's info() method or extracted from a package.xml or a composer.json file
+     *
+     * @param string $plugin_name Plugin name
      *
-     * @param string Plugin name
      * @return array Meta information about a plugin or False if plugin was not found
      */
     public function get_info($plugin_name)
     {
-        static $composer_lock, $license_uris = array(
+        static $composer_lock, $license_uris = [
             'Apache'       => 'http://www.apache.org/licenses/LICENSE-2.0.html',
             'Apache-2'     => 'http://www.apache.org/licenses/LICENSE-2.0.html',
             'Apache-1'     => 'http://www.apache.org/licenses/LICENSE-1.0',
             'Apache-1.1'   => 'http://www.apache.org/licenses/LICENSE-1.1',
             'GPL'          => 'http://www.gnu.org/licenses/gpl.html',
-            'GPLv2'        => 'http://www.gnu.org/licenses/gpl-2.0.html',
             'GPL-2.0'      => 'http://www.gnu.org/licenses/gpl-2.0.html',
-            'GPLv3'        => 'http://www.gnu.org/licenses/gpl-3.0.html',
-            'GPLv3+'       => 'http://www.gnu.org/licenses/gpl-3.0.html',
+            'GPL-2.0+'     => 'http://www.gnu.org/licenses/gpl.html',
             'GPL-3.0'      => 'http://www.gnu.org/licenses/gpl-3.0.html',
             'GPL-3.0+'     => 'http://www.gnu.org/licenses/gpl.html',
-            'GPL-2.0+'     => 'http://www.gnu.org/licenses/gpl.html',
-            'AGPLv3'       => 'http://www.gnu.org/licenses/agpl.html',
-            'AGPLv3+'      => 'http://www.gnu.org/licenses/agpl.html',
             'AGPL-3.0'     => 'http://www.gnu.org/licenses/agpl.html',
+            'AGPL-3.0+'     => 'http://www.gnu.org/licenses/agpl.html',
             'LGPL'         => 'http://www.gnu.org/licenses/lgpl.html',
-            'LGPLv2'       => 'http://www.gnu.org/licenses/lgpl-2.0.html',
-            'LGPLv2.1'     => 'http://www.gnu.org/licenses/lgpl-2.1.html',
-            'LGPLv3'       => 'http://www.gnu.org/licenses/lgpl.html',
             'LGPL-2.0'     => 'http://www.gnu.org/licenses/lgpl-2.0.html',
             'LGPL-2.1'     => 'http://www.gnu.org/licenses/lgpl-2.1.html',
             'LGPL-3.0'     => 'http://www.gnu.org/licenses/lgpl.html',
@@ -284,7 +302,7 @@ class rcube_plugin_api
             'PHP-3'        => 'http://www.php.net/license/3_01.txt',
             'PHP-3.0'      => 'http://www.php.net/license/3_0.txt',
             'PHP-3.01'     => 'http://www.php.net/license/3_01.txt',
-        );
+        ];
 
         $dir  = dir($this->dir);
         $fn   = unslashify($dir->path) . "/$plugin_name/$plugin_name.php";
@@ -292,9 +310,13 @@ class rcube_plugin_api
 
         // Validate the plugin name to prevent from path traversal
         if (preg_match('/[^a-zA-Z0-9_-]/', $plugin_name)) {
-            rcube::raise_error(array('code' => 520,
-                    'file' => __FILE__, 'line' => __LINE__,
-                    'message' => "Invalid plugin name: $plugin_name"), true, false);
+            rcube::raise_error([
+                    'code' => 520, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid plugin name: $plugin_name"
+                ],
+                true, false
+            );
+
             return false;
         }
 
@@ -315,17 +337,32 @@ class rcube_plugin_api
         if (!$info) {
             $composer = INSTALL_PATH . "/plugins/$plugin_name/composer.json";
             if (is_readable($composer) && ($json = @json_decode(file_get_contents($composer), true))) {
+                // Build list of plugins required
+                $require = [];
+                foreach (array_keys((array) $json['require']) as $dname) {
+                    if (!preg_match('|^([^/]+)/([a-zA-Z0-9_-]+)$|', $dname, $m)) {
+                        continue;
+                    }
+
+                    $vendor = $m[1];
+                    $name   = $m[2];
+
+                    if ($name != 'plugin-installer' && $vendor != 'pear' && $vendor != 'pear-pear') {
+                        $dpath = unslashify($dir->path) . "/$name/$name.php";
+                        if (is_readable($dpath)) {
+                            $require[] = $name;
+                        }
+                    }
+                }
+
                 list($info['vendor'], $info['name']) = explode('/', $json['name']);
                 $info['version'] = $json['version'];
                 $info['license'] = $json['license'];
-                $info['uri']     = $json['homepage'];
-                $info['require'] = array_filter(array_keys((array)$json['require']), function($pname) {
-                    if (strpos($pname, '/') == false) {
-                        return false;
-                    }
-                    list($vendor, $name) = explode('/', $pname);
-                    return !($name == 'plugin-installer' || $vendor == 'pear-pear');
-                });
+                $info['require'] = $require;
+
+                if (!empty($json['homepage'])) {
+                    $info['uri'] = $json['homepage'];
+                }
             }
 
             // read local composer.lock file (once)
@@ -339,7 +376,10 @@ class rcube_plugin_api
             }
 
             // load additional information from local composer.lock file
-            if ($lock = $composer_lock['installed'][$json['name']]) {
+            if (!empty($json['name']) && !empty($composer_lock['installed'])
+                && !empty($composer_lock['installed'][$json['name']])
+            ) {
+                $lock            = $composer_lock['installed'][$json['name']];
                 $info['version'] = $lock['version'];
                 $info['uri']     = $lock['homepage'] ?: $lock['source']['uri'];
                 $info['src_uri'] = $lock['dist']['uri'] ?: $lock['source']['uri'];
@@ -356,14 +396,14 @@ class rcube_plugin_api
                 $xpath->registerNamespace('rc', "http://pear.php.net/dtd/package-2.0");
 
                 // XPaths of plugin metadata elements
-                $metadata = array(
-                    'name'    => 'string(//rc:package/rc:name)',
-                    'version' => 'string(//rc:package/rc:version/rc:release)',
-                    'license' => 'string(//rc:package/rc:license)',
+                $metadata = [
+                    'name'        => 'string(//rc:package/rc:name)',
+                    'version'     => 'string(//rc:package/rc:version/rc:release)',
+                    'license'     => 'string(//rc:package/rc:license)',
                     'license_uri' => 'string(//rc:package/rc:license/@uri)',
-                    'src_uri' => 'string(//rc:package/rc:srcuri)',
-                    'uri'     => 'string(//rc:package/rc:uri)',
-                );
+                    'src_uri'     => 'string(//rc:package/rc:srcuri)',
+                    'uri'         => 'string(//rc:package/rc:uri)',
+                ];
 
                 foreach ($metadata as $key => $path) {
                     $info[$key] = $xpath->evaluate($path);
@@ -380,10 +420,18 @@ class rcube_plugin_api
 
         // At least provide the name
         if (!$info && class_exists($plugin_name)) {
-            $info = array('name' => $plugin_name, 'version' => '--');
+            $info = ['name' => $plugin_name, 'version' => '--'];
         }
-        else if ($info['license'] && empty($info['license_uri']) && ($license_uri = $license_uris[$info['license']])) {
-            $info['license_uri'] = $license_uri;
+        else if (!empty($info['license'])) {
+            // Convert license identifier to something shorter
+            if (preg_match('/^([ALGP]+)[-v]([0-9.]+)(\+|-or-later)?/', $info['license'], $matches)) {
+                $info['license'] = $matches[1] . '-' . sprintf('%.1f', $matches[2])
+                    . (!empty($matches[3]) ? '+' : '');
+            }
+
+            if (empty($info['license_uri']) && !empty($license_uris[$info['license']])) {
+                $info['license_uri'] = $license_uris[$info['license']];
+            }
         }
 
         return $info;
@@ -392,36 +440,45 @@ class rcube_plugin_api
     /**
      * Allows a plugin object to register a callback for a certain hook
      *
-     * @param string $hook Hook name
-     * @param mixed  $callback String with global function name or array($obj, 'methodname')
+     * @param string   $hook     Hook name
+     * @param callback $callback A callback function
      */
     public function register_hook($hook, $callback)
     {
         if (is_callable($callback)) {
             if (isset($this->deprecated_hooks[$hook])) {
-                rcube::raise_error(array('code' => 522, 'type' => 'php',
-                    'file' => __FILE__, 'line' => __LINE__,
-                    'message' => "Deprecated hook name. "
-                        . $hook . ' -> ' . $this->deprecated_hooks[$hook]), true, false);
+                rcube::raise_error([
+                        'code' => 522, 'file' => __FILE__, 'line' => __LINE__,
+                        'message' => "Deprecated hook name. "
+                            . $hook . ' -> ' . $this->deprecated_hooks[$hook]
+                    ], true, false
+                );
                 $hook = $this->deprecated_hooks[$hook];
             }
             $this->handlers[$hook][] = $callback;
         }
         else {
-            rcube::raise_error(array('code' => 521, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Invalid callback function for $hook"), true, false);
+            rcube::raise_error([
+                    'code' => 521, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid callback function for $hook"
+                ],
+                true, false
+            );
         }
     }
 
     /**
      * Allow a plugin object to unregister a callback.
      *
-     * @param string $hook Hook name
-     * @param mixed  $callback String with global function name or array($obj, 'methodname')
+     * @param string   $hook     Hook name
+     * @param callback $callback A callback function
      */
     public function unregister_hook($hook, $callback)
     {
+        if (empty($this->handlers[$hook])) {
+            return;
+        }
+
         $callback_id = array_search($callback, (array) $this->handlers[$hook]);
         if ($callback_id !== false) {
             // array_splice() removes the element and re-indexes keys
@@ -435,19 +492,19 @@ class rcube_plugin_api
      * This is called from the application and executes all registered handlers
      *
      * @param string $hook Hook name
-     * @param array $args Named arguments (key->value pairs)
+     * @param array  $args Named arguments (key->value pairs)
      *
      * @return array The (probably) altered hook arguments
      */
-    public function exec_hook($hook, $args = array())
+    public function exec_hook($hook, $args = [])
     {
         if (!is_array($args)) {
-            $args = array('arg' => $args);
+            $args = ['arg' => $args];
         }
 
         // TODO: avoid recursion by checking in_array($hook, $this->exec_stack) ?
 
-        $args += array('abort' => false);
+        $args += ['abort' => false];
         array_push($this->exec_stack, $hook);
 
         // Use for loop here, so handlers added in the hook will be executed too
@@ -458,7 +515,7 @@ class rcube_plugin_api
                     $args = $ret + $args;
                 }
 
-                if ($args['break']) {
+                if (!empty($args['break'])) {
                     break;
                 }
             }
@@ -471,18 +528,20 @@ class rcube_plugin_api
     /**
      * Let a plugin register a handler for a specific request
      *
-     * @param string $action   Action name (_task=mail&_action=plugin.foo)
-     * @param string $owner    Plugin name that registers this action
-     * @param mixed  $callback Callback: string with global function name or array($obj, 'methodname')
-     * @param string $task     Task name registered by this plugin
+     * @param string   $action   Action name (_task=mail&_action=plugin.foo)
+     * @param string   $owner    Plugin name that registers this action
+     * @param callback $callback A callback function
+     * @param string   $task     Task name registered by this plugin
      */
     public function register_action($action, $owner, $callback, $task = null)
     {
         // check action name
-        if ($task)
+        if ($task) {
             $action = $task.'.'.$action;
-        else if (strpos($action, 'plugin.') !== 0)
+        }
+        else if (strpos($action, 'plugin.') !== 0) {
             $action = 'plugin.'.$action;
+        }
 
         // can register action only if it's not taken or registered by myself
         if (!isset($this->actionmap[$action]) || $this->actionmap[$action] == $owner) {
@@ -490,10 +549,12 @@ class rcube_plugin_api
             $this->actionmap[$action] = $owner;
         }
         else {
-            rcube::raise_error(array('code' => 523, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Cannot register action $action;"
-                    ." already taken by another plugin"), true, false);
+            rcube::raise_error([
+                    'code' => 523, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Cannot register action $action; already taken by another plugin"
+                ],
+                true, false
+            );
         }
     }
 
@@ -509,18 +570,21 @@ class rcube_plugin_api
             call_user_func($this->actions[$action]);
         }
         else if (rcube::get_instance()->action != 'refresh') {
-            rcube::raise_error(array('code' => 524, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "No handler found for action $action"), true, true);
+            rcube::raise_error([
+                    'code' => 524, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "No handler found for action $action"
+                ],
+                true, true
+            );
         }
     }
 
     /**
      * Register a handler function for template objects
      *
-     * @param string $name Object name
-     * @param string $owner Plugin name that registers this action
-     * @param mixed  $callback Callback: string with global function name or array($obj, 'methodname')
+     * @param string   $name     Object name
+     * @param string   $owner    Plugin name that registers this action
+     * @param callback $callback A callback function
      */
     public function register_handler($name, $owner, $callback)
     {
@@ -537,17 +601,20 @@ class rcube_plugin_api
             $this->objectsmap[$name] = $owner;
         }
         else {
-            rcube::raise_error(array('code' => 525, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Cannot register template handler $name;"
-                    ." already taken by another plugin or no output object available"), true, false);
+            rcube::raise_error([
+                    'code' => 525, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Cannot register template handler $name;"
+                        ." already taken by another plugin or no output object available"
+                ],
+                true, false
+            );
         }
     }
 
     /**
      * Register this plugin to be responsible for a specific task
      *
-     * @param string $task Task name (only characters [a-z0-9_-] are allowed)
+     * @param string $task  Task name (only characters [a-z0-9_-] are allowed)
      * @param string $owner Plugin name that registers this action
      */
     public function register_task($task, $owner)
@@ -558,16 +625,22 @@ class rcube_plugin_api
         }
 
         if ($task != asciiwords($task, true)) {
-            rcube::raise_error(array('code' => 526, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Invalid task name: $task."
-                    ." Only characters [a-z0-9_.-] are allowed"), true, false);
+            rcube::raise_error([
+                    'code' => 526, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid task name: $task."
+                        ." Only characters [a-z0-9_.-] are allowed"
+                ],
+                true, false
+            );
         }
         else if (in_array($task, rcmail::$main_tasks)) {
-            rcube::raise_error(array('code' => 526, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Cannot register taks $task;"
-                    ." already taken by another plugin or the application itself"), true, false);
+            rcube::raise_error([
+                    'code' => 526, 'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Cannot register task $task;"
+                        ." already taken by another plugin or the application itself"
+                ],
+                true, false
+            );
         }
         else {
             $this->tasks[$task] = $owner;
@@ -583,11 +656,11 @@ class rcube_plugin_api
      *
      * @param string $task Task name
      *
-     * @return boolean True if registered, otherwise false
+     * @return bool True if registered, otherwise false
      */
     public function is_plugin_task($task)
     {
-        return $this->tasks[$task] ? true : false;
+        return !empty($this->tasks[$task]);
     }
 
     /**
@@ -596,7 +669,7 @@ class rcube_plugin_api
      *
      * @param string $hook Hook to check (optional)
      *
-     * @return boolean True if any/the given hook is currently processed, otherwise false
+     * @return bool True if any/the given hook is currently processed, otherwise false
      */
     public function is_processing($hook = null)
     {
@@ -612,8 +685,7 @@ class rcube_plugin_api
     {
         if (is_object($this->output) && $this->output->type == 'html') {
             $src = $this->resource_url($fn);
-            $this->output->add_header(html::tag('script',
-                array('type' => "text/javascript", 'src' => $src)));
+            $this->output->include_script($src, 'head_bottom', false);
         }
     }
 
@@ -625,6 +697,32 @@ class rcube_plugin_api
     public function include_stylesheet($fn)
     {
         if (is_object($this->output) && $this->output->type == 'html') {
+            if ($fn[0] != '/' && !preg_match('|^https?://|i', $fn)) {
+                $rcube      = rcube::get_instance();
+                $devel_mode = $rcube->config->get('devel_mode');
+                $assets_dir = $rcube->config->get('assets_dir');
+                $path       = unslashify($assets_dir ?: RCUBE_INSTALL_PATH);
+                $dir        = $path . (strpos($fn, "plugins/") === false ? '/plugins' : '');
+
+                // Prefer .less files in devel_mode (assume less.js is loaded)
+                if ($devel_mode) {
+                    $less = preg_replace('/\.css$/i', '.less', $fn);
+                    if ($less != $fn && is_file("$dir/$less")) {
+                        $fn = $less;
+                    }
+                }
+                else if (!preg_match('/\.min\.css$/', $fn)) {
+                    $min = preg_replace('/\.css$/i', '.min.css', $fn);
+                    if (is_file("$dir/$min")) {
+                        $fn = $min;
+                    }
+                }
+
+                if (!is_file("$dir/$fn")) {
+                    return;
+                }
+            }
+
             $src = $this->resource_url($fn);
             $this->output->include_css($src);
         }
@@ -633,11 +731,15 @@ class rcube_plugin_api
     /**
      * Save the given HTML content to be added to a template container
      *
-     * @param string $html HTML content
+     * @param string $html      HTML content
      * @param string $container Template container identifier
      */
     public function add_content($html, $container)
     {
+        if (!isset($this->template_contents[$container])) {
+            $this->template_contents[$container] = '';
+        }
+
         $this->template_contents[$container] .= $html . "\n";
     }
 
@@ -654,36 +756,47 @@ class rcube_plugin_api
     /**
      * Returns loaded plugin
      *
-     * @return rcube_plugin Plugin instance
+     * @return rcube_plugin|null Plugin instance
      */
     public function get_plugin($name)
     {
-        return $this->plugins[$name];
+        return !empty($this->plugins[$name]) ? $this->plugins[$name] : null;
     }
 
     /**
      * Callback for template_container hooks
      *
-     * @param array $attrib
+     * @param array $attrib Container attributes
+     *
      * @return array
      */
     protected function template_container_hook($attrib)
     {
-        $container = $attrib['name'];
-        return array('content' => $attrib['content'] . $this->template_contents[$container]);
+        $container     = $attrib['name'];
+        $content       = isset($attrib['content']) ? $attrib['content'] : '';
+
+        if (isset($this->template_contents[$container])) {
+            $content .= $this->template_contents[$container];
+        }
+
+        return ['content' => $content];
     }
 
     /**
      * Make the given file name link into the plugins directory
      *
      * @param string $fn Filename
+     *
      * @return string
      */
     protected function resource_url($fn)
     {
-        if ($fn[0] != '/' && !preg_match('|^https?://|i', $fn))
+        // pattern "skins/" used to identify plugin resources loaded from the core skin folder
+        if ($fn[0] != '/' && !preg_match('#^(https?://|skins/)#i', $fn)) {
             return $this->url . $fn;
-        else
+        }
+        else {
             return $fn;
+        }
     }
 }
