@@ -5,7 +5,7 @@
  *
  * @package Tests
  */
-class Framework_Washtml extends PHPUnit_Framework_TestCase
+class Framework_Washtml extends PHPUnit\Framework\TestCase
 {
     /**
      * A helper method to remove comments added by rcube_washtml
@@ -15,7 +15,6 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         return preg_replace('/<!-- [a-z]+ (ignored|not allowed) -->/', '', $html);
     }
 
-
     /**
      * Test the elimination of some XSS vulnerabilities
      */
@@ -70,6 +69,23 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         $this->assertNotRegExp('/javascript:/', $washed, "javascript: in area href");
     }
 
+    /**
+     * Test removing of object tag, but keeping innocent children
+     */
+    function test_object()
+    {
+        $html = "<div>\n<object data=\"move.swf\" type=\"application/x-shockwave-flash\">\n"
+               ."<param name=\"foo\" value=\"bar\">\n"
+               ."<p>This alternative text should survive</p>"
+               ."</object>\n</div>";
+        $washer = new rcube_washtml;
+        $washed = $washer->wash($html);
+
+        $this->assertNotRegExp('/<\/?object/', $washed, "Remove object tag");
+        $this->assertNotRegExp('/<param/', $washed, "Remove param tag");
+        $this->assertRegExp('/<p>/', $washed, "Keep embedded tags");
+    }
+
     /**
      * Test handling HTML comments
      */
@@ -139,36 +155,36 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
      */
     function test_lists()
     {
-        $data = array(
-            array(
+        $data = [
+            [
                 "<ol><li>First</li><li>Second</li><ul><li>First sub</li></ul><li>Third</li></ol>",
                 "<ol><li>First</li><li>Second<ul><li>First sub</li></ul></li><li>Third</li></ol>"
-            ),
-            array(
+            ],
+            [
                 "<ol><li>First<ul><li>First sub</li></ul></li></ol>",
                 "<ol><li>First<ul><li>First sub</li></ul></li></ol>",
-            ),
-            array(
+            ],
+            [
                 "<ol><li>First<ol><li>First sub</li></ol></li></ol>",
                 "<ol><li>First<ol><li>First sub</li></ol></li></ol>",
-            ),
-            array(
+            ],
+            [
                 "<ul><li>First</li><ul><li>First sub</li><ul><li>sub sub</li></ul></ul><li></li></ul>",
                 "<ul><li>First<ul><li>First sub<ul><li>sub sub</li></ul></li></ul></li><li></li></ul>",
-            ),
-            array(
+            ],
+            [
                 "<ul><li>First</li><li>second</li><ul><ul><li>sub sub</li></ul></ul></ul>",
                 "<ul><li>First</li><li>second<ul><ul><li>sub sub</li></ul></ul></li></ul>",
-            ),
-            array(
+            ],
+            [
                 "<ol><ol><ol></ol></ol></ol>",
                 "<ol><ol><ol></ol></ol></ol>",
-            ),
-            array(
+            ],
+            [
                 "<div><ol><ol><ol></ol></ol></ol></div>",
                 "<div><ol><ol><ol></ol></ol></ol></div>",
-            ),
-        );
+            ],
+        ];
 
         foreach ($data as $element) {
             rcube_washtml::fix_broken_lists($element[0]);
@@ -222,7 +238,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             <body bgcolor=\"#fff\" text=\"#000\" background=\"#test\" link=\"#111\" alink=\"#222\" vlink=\"#333\">
             </body></html>";
 
-        $washer = new rcube_washtml(array('html_elements' => array('body')));
+        $washer = new rcube_washtml(['html_elements' => ['body']]);
         $washed = $washer->wash($html);
 
         $this->assertRegExp('|bgcolor="#fff"|', $washed, "Body bgcolor attribute");
@@ -326,11 +342,11 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         return [
             [
                 '<head xmlns="&quot;&gt;&lt;script&gt;alert(document.domain)&lt;/script&gt;"><svg></svg></head>',
-                '<!-- html ignored --><!-- head ignored --><svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
+                '<svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
             ],
             [
                 '<head xmlns="&quot; onload=&quot;alert(document.domain)">Hello victim!<svg></svg></head>',
-                '<!-- html ignored --><!-- head ignored -->Hello victim!<svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
+                'Hello victim!<svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
             ],
             [
                 '<p>Hello victim!<svg xmlns="&quot; onload=&quot;alert(document.domain)"></svg></p>',
@@ -338,7 +354,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             ],
             [
                 '<html><p>Hello victim!<svg xmlns="&quot; onload=&quot;alert(document.domain)"></svg></p>',
-                '<!-- html ignored --><!-- body ignored --><p>Hello victim!<svg xmlns="http://www.w3.org/1999/xhtml"></svg></p>'
+                '<p>Hello victim!<svg xmlns="http://www.w3.org/1999/xhtml"></svg></p>'
             ],
             [
                 '<svg xmlns="&quot; onload=&quot;alert(document.domain)" />',
@@ -346,7 +362,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             ],
             [
                 '<html><svg xmlns="&quot; onload=&quot;alert(document.domain)" />',
-                '<!-- html ignored --><!-- body ignored --><svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
+                '<svg xmlns="http://www.w3.org/1999/xhtml"></svg>'
             ],
             [
                 '<svg><a xlink:href="javascript:alert(1)"><text x="20" y="20">XSS</text></a></svg>',
@@ -354,7 +370,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             ],
             [
                 '<html><svg><a xlink:href="javascript:alert(1)"><text x="20" y="20">XSS</text></a></svg>',
-                '<!-- html ignored --><!-- body ignored --><svg xmlns="http://www.w3.org/1999/xhtml"><a x-washed="xlink:href"><text x="20" y="20">XSS</text></a></svg>'
+                '<svg xmlns="http://www.w3.org/1999/xhtml"><a x-washed="xlink:href"><text x="20" y="20">XSS</text></a></svg>'
             ],
             [
                 '<svg><animate xlink:href="#xss" attributeName="href" values="javascript:alert(1)" />'
@@ -364,7 +380,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             [
                 '<html><svg><animate xlink:href="#xss" attributeName="href" values="javascript:alert(1)" />'
                     . '<a id="xss"><text x="20" y="20">XSS</text></a></svg>',
-                '<!-- html ignored --><!-- body ignored --><svg xmlns="http://www.w3.org/1999/xhtml">'
+                '<svg xmlns="http://www.w3.org/1999/xhtml">'
                     . '<!-- animate blocked --><a id="xss"><text x="20" y="20">XSS</text></a></svg>',
             ],
             [
@@ -399,25 +415,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
             ],
             [
                 '<svg><script href="data:text/javascript,alert(1)" /><text x="20" y="20">XSS</text></svg>',
-                '<svg><!-- script not allowed --><text x="20" y="20">XSS</text></svg>'
-            ],
-            [
-                '<html><svg><use href="data:image/s vg+xml;base64,' // space
-                    . 'PHN2ZyBpZD0ieCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4gPGltYWdlIGhy'
-                    . 'ZWY9IngiIG9uZXJyb3I9ImFsZXJ0KCcxJykiLz48L3N2Zz4=#x"></svg></html>',
-                '<svg xmlns="http://www.w3.org/1999/xhtml"><use x-washed="href"></use></svg>'
-            ],
-            [
-                '<html><svg><use href="data:image/s' . "\n" . 'vg+xml;base64,' // new-line
-                    . 'PHN2ZyBpZD0ieCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4gPGltYWdlIGhy'
-                    . 'ZWY9IngiIG9uZXJyb3I9ImFsZXJ0KCcxJykiLz48L3N2Zz4=#x"></svg></html>',
-                '<svg xmlns="http://www.w3.org/1999/xhtml"><use x-washed="href"></use></svg>'
-            ],
-            [
-                '<html><svg><use href="data:image/s	vg+xml;base64,' // tab
-                    . 'PHN2ZyBpZD0ieCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4gPGltYWdlIGhy'
-                    . 'ZWY9IngiIG9uZXJyb3I9ImFsZXJ0KCcxJykiLz48L3N2Zz4=#x"></svg></html>',
-                '<svg xmlns="http://www.w3.org/1999/xhtml"><use x-washed="href"></use></svg>'
+                '<svg><text x="20" y="20">XSS</text></svg>'
             ],
         ];
     }
@@ -432,7 +430,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         $washer = new rcube_washtml;
         $washed = $washer->wash($input);
 
-        $this->assertSame($expected, $washed, "SVG content");
+        $this->assertSame($expected, $this->cleanupResult($washed), "SVG content");
     }
 
     /**
@@ -443,47 +441,43 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         return [
             [
                 '<html><base href="javascript:/a/-alert(1)///////"><a href="../lol/safari.html">test</a>',
-                '<!-- html ignored --><body><!-- base ignored --><a x-washed="href">test</a></body>'
+                '<body><a x-washed="href">test</a></body>'
             ],
             [
                 '<html><math><x href="javascript:alert(1)">blah</x>',
-                '<!-- html ignored --><body><math><!-- x ignored -->blah</math></body>'
+                '<body><math>blah</math></body>'
             ],
             [
                 '<html><a href="j&#x61vascript:alert(1)">XSS</a>',
-                '<!-- html ignored --><body><a x-washed="href">XSS</a></body>'
+                '<body><a x-washed="href">XSS</a></body>'
             ],
             [
                 '<html><a href="&#x6a avascript:alert(1)">XSS</a>',
-                '<!-- html ignored --><body><a x-washed="href">XSS</a></body>'
+                '<body><a x-washed="href">XSS</a></body>'
             ],
             [
                 '<html><a href="&#x6a avascript:alert(1)">XSS</a>',
-                '<!-- html ignored --><body><a x-washed="href">XSS</a></body>'
+                '<body><a x-washed="href">XSS</a></body>'
             ],
             [
                 '<html><body background="javascript:alert(1)">',
-                '<!-- html ignored --><body x-washed="background"></body>'
-            ],
-            [
-                '<html><body><img fill=\'asd:url(#asd)" src="x" onerror="alert(1)\' />',
-                '<body><img fill="asd:url(#asd)&quot; src=&quot;x&quot; onerror=&quot;alert(1)" /></body>'
+                '<body x-washed="background"></body>'
             ],
             [
                 '<html><math href="javascript:alert(location);"><mi>clickme</mi></math>',
-                '<!-- html ignored --><body><math x-washed="href"><mi>clickme</mi></math></body>',
+                '<body><math x-washed="href"><mi>clickme</mi></math></body>',
             ],
             [
                 '<html><math><mstyle href="javascript:alert(location);"><mi>clickme</mi></mstyle></math>',
-                '<!-- html ignored --><body><math><mstyle x-washed="href"><mi>clickme</mi></mstyle></math></body>',
+                '<body><math><mstyle x-washed="href"><mi>clickme</mi></mstyle></math></body>',
             ],
             [
                 '<html><math><msubsup href="javascript:alert(location);"><mi>clickme</mi></msubsup></math>',
-                '<!-- html ignored --><body><math><msubsup x-washed="href"><mi>clickme</mi></msubsup></math></body>',
+                '<body><math><msubsup x-washed="href"><mi>clickme</mi></msubsup></math></body>',
             ],
             [
                 '<html><math><ms HREF="javascript:alert(location);">clickme</ms></math>',
-                '<!-- html ignored --><body><math><ms x-washed="href">clickme</ms></math></body>',
+                '<body><math><ms x-washed="href">clickme</ms></math></body>',
             ],
         ];
     }
@@ -498,7 +492,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         $washer = new rcube_washtml(['allow_remote' => true, 'html_elements' => ['body']]);
         $washed = $washer->wash($input);
 
-        $this->assertSame($expected, $washed, "XSS issues");
+        $this->assertSame($expected, $this->cleanupResult($washed), "XSS issues");
     }
 
     /**
@@ -590,13 +584,13 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
      */
     function test_extlinks()
     {
-        $html = array(
-            array("<link href=\"http://TRACKING_URL/\">", true),
-            array("<link href=\"src:abc\">", false),
-            array("<img src=\"http://TRACKING_URL/\">", true),
-            array("<img src=\"data:image\">", false),
-            array('<p style="backgr\\ound-image: \\ur\\l(\'http://TRACKING_URL\')"></p>', true),
-        );
+        $html = [
+            ["<link href=\"http://TRACKING_URL/\">", true],
+            ["<link href=\"src:abc\">", false],
+            ["<img src=\"http://TRACKING_URL/\">", true],
+            ["<img src=\"data:image\">", false],
+            ['<p style="backgr\\ound-image: \\ur\\l(\'http://TRACKING_URL\')"></p>', true],
+        ];
 
         foreach ($html as $item) {
             $washer = new rcube_washtml;
@@ -606,7 +600,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
         }
 
         foreach ($html as $item) {
-            $washer = new rcube_washtml(array('allow_remote' => true));
+            $washer = new rcube_washtml(['allow_remote' => true]);
             $washed = $washer->wash($item[0]);
 
             $this->assertFalse($washer->extlinks);
@@ -629,7 +623,7 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
      */
     function test_css_prefix()
     {
-        $washer = new rcube_washtml(array('css_prefix' => 'test'));
+        $washer = new rcube_washtml(['css_prefix' => 'test']);
 
         $html   = '<p id="my-id">'
             . '<label for="my-other-id" class="my-class1 my-class2">test</label>'
@@ -715,4 +709,62 @@ class Framework_Washtml extends PHPUnit_Framework_TestCase
 
         $this->assertTrue(strpos($washed, '<script>') === false, "CDATA content");
     }
+
+    /**
+     * Test URI base resolving in HTML messages
+     */
+    function test_resolve_base()
+    {
+        $html = file_get_contents(TESTS_DIR . 'src/htmlbase.txt');
+        $html = rcube_washtml::resolve_base($html);
+
+        $this->assertRegExp('|src="http://alec\.pl/dir/img1\.gif"|', $html, "URI base resolving [1]");
+        $this->assertRegExp('|src="http://alec\.pl/dir/img2\.gif"|', $html, "URI base resolving [2]");
+        $this->assertRegExp('|src="http://alec\.pl/img3\.gif"|', $html, "URI base resolving [3]");
+
+        // base resolving exceptions
+        $this->assertRegExp('|src="cid:theCID"|', $html, "URI base resolving exception [1]");
+        $this->assertRegExp('|src="http://other\.domain\.tld/img3\.gif"|', $html, "URI base resolving exception [2]");
+    }
+
+    /**
+     * Test workaround for HTML5 bug (#7356)
+     */
+    function test_table_bug7356()
+    {
+        $html = '
+<table id="t1">
+  <tr>
+    <td>
+      <table id="t2">
+        <tr>
+        <tr>
+          <td></td>
+        </tr>
+        </tr>
+      </table>
+    </td>
+  </tr>
+  <tr><td></td></tr>
+</table>';
+
+        $expected = '
+<table id="t1">
+  <tr>
+    <td>
+      <table id="t2">
+        <tr>
+          <td></td>
+        </tr>
+      </table>
+    </td>
+  </tr>
+  <tr><td></td></tr>
+</table>';
+
+        $washer = new rcube_washtml;
+        $washed = $this->cleanupResult($washer->wash($html));
+
+        $this->assertSame(trim($expected), $washed);
+    }
 }
